   /* AgeDependBP.c */
   /* Age-dependent branching process with Type I,II,or III life time :
      Distributions of the age and the life remaining */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

double frand(void)
{
 return 1.0-(double)rand()/(RAND_MAX+1.0);
}

#define maxsize 30001   /* Maximum population size */

double mean(p)
 double p[5];
 {
  double mu=0.0;
  int i;
  for(i=1;i<5;++i) mu +=i*p[i];
  return mu;
 }
int offspring(p,v)
 double p[5],v;
 {
  int i=0;
  double u=p[0];
  while ((i<4)&&(v>=u)) {
   ++i;u+=p[i];
  }
  return i;
 }
float minlist(BDL)
 float BDL[maxsize][2];
 {
  long int i=1;
  float a=BDL[0][1];
  while (i<maxsize && BDL[i][1]>0.0) {
   if (a>BDL[i][1]) a=BDL[i][1];
   ++i;
  }
  return a;
 }
void renew(BDL,children,lives)
 float BDL[maxsize][2],lives[4];
 int children;
 {
  int k;
  long int i=1,j=0;
  float a=BDL[0][1];
  while (i<maxsize && BDL[i][1]>0.0) {
   if (a>BDL[i][1]) {a=BDL[i][1];j=i;}
   ++i;
  }
  if (i<maxsize-children+1) {
   if (children>1) {
    while (i>j) {
     BDL[i-1+children][0]=BDL[i][0];BDL[i-1+children][1]=BDL[i][1];
     --i;
    }
   }
   if (children>0) {
    for (k=0;k<children;k++) {BDL[j+k][0]=a;BDL[j+k][1]=a+lives[k];}
   }
   else {
    while (BDL[j][1]>0.0) {
     BDL[j][0]=BDL[j+1][0];BDL[j][1]=BDL[j+1][1];
     ++j;
    }
   }
  }
 }

main(void)
{
 int j,k;
 long int i,seed,
          x; /* size at each time of the process */
 float a, /* exponent of life time distribution */
       simtime, /* time to be simulated */
       BDL[maxsize][2], /* list of birth and death times of a current
population */
       lives[4], /* list of life times of children */
       t; /* time when birth or death occurs */
 double p[5]; /* probability of giving birth to babies */
 char *file1 = "datax.dat",*file2 = "samplex.dat",
      *file3 = "liferemainx.dat",*file4 = "agex.dat";
 FILE *fp1,*fp2,*fp3,*fp4;

   /*----------------- Input initial parameters -----------------*/

 printf("Exponent a of life time dist.(a>1 for type I, a=1 for type II, and 0<a<1 for type III)=");
 scanf("%f",&a);
 printf("\n");
 printf(" Offspring distribution:\n\t");
 for(i=0;i<4;++i){
  printf("p(%d)=",i);
  scanf("%lf",&p[i]);
  printf("\t");
 }
 p[4]=1.0-p[0]-p[1]-p[2]-p[3];
 printf("\n");
 printf(" Mean offspring number = %4.3f\n",mean(p));
 printf("\n");
 printf(" Time to be observed (positive number) = ");
 scanf("%f",&simtime);
 printf("\n");
 printf(" Initial population size (any positive integer) = ");
 scanf("%d",&x);
 printf("\n");
 printf(" Random number seed (any integer) = ");
 scanf("%d",&seed);
 printf("\n\n");

  /* ---- Print initial parameters in a file "datax.dat" ----*/

 fp1=fopen(file1,"w");
 fprintf(fp1,"  << Age-dependent branching process with Type I,II,or III life time >>\n");
 fprintf(fp1,"   <Age distribution and the distribution of life remaining> \n\n");
 fprintf(fp1," Exponent of life time dist. = %4.2f\n\n",a);
 fprintf(fp1," Mean life time = %4.2f\n\n",a/(a+1.0));
 fprintf(fp1," Offspring distribution:\n");
 for(i=0;i<5;++i) fprintf(fp1,"  p(%d)=%3.3f  ",i,p[i]);
 fprintf(fp1,"\n\n");
 fprintf(fp1," Mean offspring number = %4.3f\n\n",mean(p));
 fprintf(fp1," Time to be observed = %4.2f\n\n",simtime);
 fprintf(fp1," Initial population size = %d\n\n",x);
 fprintf(fp1," Random number seed = %d\n",seed);
 fclose(fp1);

  /* Start simulation to save the results in files "samplex.dat",
  "liferemainx.dat" and  "agex.dat"*/

 fp2=fopen(file2,"w");
 srand(seed); /* giving the seed of frand */
 fprintf(fp2,"%5.3f\t %10d\n",0.0,x);
 for (i=0;i<maxsize;++i) {
  BDL[i][0]=(float)100000.0;BDL[i][1]=(float)0.0;
 }
 for (i=0;i<x;++i) {
  BDL[i][0]=(float)0.0;BDL[i][1]=(float)pow(frand(),1.0/a);
 }
 for (j=0;j<4;++j) lives[j]=(float)0.0;
 t=minlist(BDL);
 while ((t<simtime)&&(x!=0)&&(x<maxsize)) {
  k=offspring(p,frand());
  for (j=0;j<k;++j) lives[j]=(float)pow(frand(),1.0/a);
  x+=k-1;
  fprintf(fp2,"%5.3f\t %10d\n",t,x);
  renew(BDL,k,lives);t=minlist(BDL);
 }
 fclose(fp2);
 fp3=fopen(file3,"w");fp4=fopen(file4,"w");
 i=1;
 while (i<maxsize && BDL[i][1]>0.0) {
  fprintf(fp3,"%f\n",BDL[i][1]-simtime);
  fprintf(fp4,"%f\n",simtime-BDL[i][0]);
  ++i;
 }
 fclose(fp3);fclose(fp4);
}
