   /* AvrCBP.c */
   /* Averaged continuous time branching process */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

double frand(void)
{
 return 1.0-(double)rand()/(RAND_MAX+1.0);
}

double mean(p)
 double p[5];
 {
  double mu=0.0;
   int i;
   for(i=1;i<5;++i) mu +=i*p[i];
   return mu;
 }

int offspring(p,v)
 double p[5],v;
 {
  int i=0;
  double u=p[0];
  while (i<4 && v>=u) {
   ++i;u+=p[i];
  }
  return i;
 }

main(void)
{
  int i,n,
   sample, /* number of repetition of simulations */
   init; /* initial size */
  long int seed,
   x,y; /* size after and before each time of the process */
  unsigned long int     s[10001];
  double a, /* parameter of exponential life time */
   p[5], /* probability of giving birth to babies, p[1]=0 */
   simtime, /* time to be simulated */
   d, /* atomic time for sum =simtime/10000 */
   t; /* present time */
  char *file1 = "datax.dat",*file2 = "averagex.dat";
  FILE *fp1,*fp2;

   /*------------- Input initial parameters --------------*/

 printf(" Rate of an exponential life time (any positive number) = ");
 scanf("%lf",&a);
 printf("\n");
 printf(" Offspring distribution:\n\t");
  printf("p(0)=");
  scanf("%lf",&p[0]);
  printf("\t");
  p[1]=0.0;
 for(i=2;i<4;++i){
  printf("p(%d)=",i);
  scanf("%lf",&p[i]);
  printf("\t");
 }
 p[4]=1.0-p[0]-p[2]-p[3];
 printf("\n");
 printf(" Mean offspring number = %4.3f\n",mean(p));
 printf("\n");
 printf(" Time to be observed (positive number) = ");
 scanf("%lf",&simtime);
 printf("\n");
 d=simtime/10000;
 printf(" Initial population size (any positive integer) = ");
 scanf("%d",&init);
 printf("\n");
 printf(" Samples = ");
 scanf("%d",&sample);
 printf("\n");
 printf(" Random number seed (any integer) = ");
 scanf("%d",&seed);
 printf("\n\n");

  /* ------- Print initial parametersin a file "datax.dat" -------*/

 fp1=fopen(file1,"w");
 fprintf(fp1,"      << Averaged continuous time branching process >>\n\n");
 fprintf(fp1," Rate of an exponential life time =%3.3f\n\n",a);
 fprintf(fp1," Offspring distribution:\n");
 for(i=0;i<5;++i) fprintf(fp1,"\tp(%d)=%3.3f",i,p[i]);
 fprintf(fp1,"\n\n");
 fprintf(fp1," Mean offspring number= %4.3f\n\n",mean(p));
 fprintf(fp1," Time to be observed =%4.3f\n\n",simtime);
 fprintf(fp1," Unit of time =%6.5f\n\n",d);
 fprintf(fp1," Initial population size = %d\n\n",init);
 fprintf(fp1," Samples = %d\n\n",sample);
 fprintf(fp1," Random number seed = %d\n",seed);
 fclose(fp1);

  /* Start simulation to save the results in a file "averagex.dat" */

 srand(seed); /* giving the seed of rand */
 for (n=1;n<10001;++n) s[n]=0;
 for (i=1;i<=sample;++i) {
  x=init;t=0.0;n=1;
  while (t<simtime && x!=0) {
   t+=-log(frand())/(a*x);
   y=x;x+=offspring(p,frand())-1;
   while (n<10001 && n*d<t) {s[n]+=y;++n;}
  }
  printf("%8d:%7d",i,y);
 }
 fp2=fopen(file2,"w");
 fprintf(fp2,"0.0\t %d\n",init);
 for(n=1;n<10001;++n)
  fprintf(fp2,"%5.3f\t %f\n",(double)n*d,(double)s[n]/(double)sample);
 fclose(fp2);
}
