   /* AvrDensityDepCBP2Mod.c */
   /* Averaged density-dependent CBP : 2-mode case */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

double frand(void)
{
 return 1.0-(double)rand()/(RAND_MAX+1.0);
}

double mean(p)
 double p[5];
 {
  double mu=0.0;
  int i;
  for(i=1;i<5;++i) mu +=i*p[i];
  return mu;
 }
int offspring(X,p,q,K,v)
 double p[5],q[5],v;
 int X,K;
 {
 int i;
 double u,r[5];
 if (X < K) for (i=0;i<5;++i) r[i]=p[i];
 else for (i=0;i<5;++i) r[i]=q[i];
 i=0;u=r[0];
 while (i<4 && v>=u) {
  ++i;u+=r[i];
  }
 return i;
 }

main(void)
{
 int i,k,n,
     sample, /* number of repetition of simulations */
     nonext[10001], /* surviving samples at each time atoms */
     init; /* initial size */
 long int seed,
          K, /* carrying capacity */
          x,y; /* size after and before each time of the process */
 unsigned long int s[10001];
 double a, /* parameter of exponential life time */
        p[5], /* prob. of giving birth to babies under K, p[1]=0 */
        q[5], /* prob. of giving birth to babies over K, q[1]=0 */
        simtime, /* time to be simulated */
        d, /* atomic time for sum =simtime/10000 */
        t; /* present time */
 char *file1 = "datax.dat",*file2 = "distx.dat",
      *file3 = "caveragex.dat",*file4 = "averagex.dat";
 FILE *fp1,*fp2,*fp3,*fp4;

   /*--------------- Input initial parameters ---------------*/

 printf(" Carrying capacity (any positive integer) = ");
 scanf("%d",&K);
 printf("\n");
 printf(" Rate of an exponential life time (any positive number) = ");
 scanf("%lf",&a);
 printf("\n");
 printf(" Offspring distribution below %d :\n\t",K);
  printf("p(0)=");
  scanf("%lf",&p[0]);
  printf("\t");
  p[1]=0.0;
 for(i=2;i<4;++i){
  printf("p(%d)=",i);
  scanf("%lf",&p[i]);
  printf("\t");
 }
 p[4]=1.0-p[0]-p[2]-p[3];
 printf("\n");
 printf(" Mean offspring number below %d = %4.3f\n",K,mean(p));
 printf("\n");
 printf(" Offspring distribution over %d :\n\t",K);
  printf("q(0)=");
  scanf("%lf",&q[0]);
  printf("\t");
  q[1]=0.0;
 for(i=2;i<4;++i){
  printf("q(%d)=",i);
  scanf("%lf",&q[i]);
  printf("\t");
 }
 q[4]=1.0-q[0]-q[2]-q[3];
 printf("\n");
 printf(" Mean offspring number over %d = %4.3f\n",K,mean(q));
 printf("\n");
 printf(" Time to be observed (positive number) = ");
 scanf("%lf",&simtime);
 printf("\n");
 d=simtime/10000;
 printf(" Initial population size (any positive integer) = ");
 scanf("%d",&init);
 printf("\n");
 printf(" Samples = ");
 scanf("%d",&sample);
 printf("\n");
 printf(" Random number seed (any integer) = ");
 scanf("%d",&seed);
 printf("\n\n");

  /* ---- Print initial parameters in a file "datax.dat" ----*/

 fp1=fopen(file1,"w");
 fprintf(fp1,"  << Averaged density-dependent CBP: 2-mode case >>\n\n");
 fprintf(fp1," Carrying capacity = %d\n\n",K);
 fprintf(fp1," Rate of an exponential life time =%3.3f\n\n",a);
 fprintf(fp1," Offspring distribution below %d:\n",K);
 for(i=0;i<5;++i) fprintf(fp1,"\tp(%d)=%3.3f",i,p[i]);
 fprintf(fp1,"\n\n");
 fprintf(fp1," Mean offspring number below %d= %4.3f\n\n",K,mean(p));
 fprintf(fp1," Offspring distribution over %d:\n",K);
 for(i=0;i<5;++i) fprintf(fp1,"\tq(%d)=%3.3f",i,q[i]);
 fprintf(fp1,"\n\n");
 fprintf(fp1," Mean offspring number over %d= %4.3f\n\n",K,mean(q));
 fprintf(fp1," Time to be observed =%4.3f\n\n",simtime);
 fprintf(fp1," Unit of time =%6.5f\n\n",d);
 fprintf(fp1," Initial population size = %d\n\n",init);
 fprintf(fp1," Samples = %d\n\n",sample);
 fprintf(fp1," Random number seed = %d\n",seed);
 fclose(fp1);

  /* Start simulation to save the results in files "distx.dat",
     "caveragex.dat" and "averagex.dat" */

 fp2=fopen(file2,"w");
 srand(seed); /* giving the seed of rand */
 for (n=1;n<10001;++n) {s[n]=0;nonext[n]=sample;}
 for (i=1;i<=sample;++i) {
  x=init;t=0.0;n=1;
  while (t<simtime && x!=0) {
   t+=-log(frand())/(a*x);
   y=x;x+=offspring(x,p,q,K,frand())-1;
   while (n<10001 && n*d<t) {s[n]+=y;++n;}
   if (x==0) {
    for (k=n;k<10001;++k) --nonext[k];
    y=0;
   }
  }
  printf("%8d:%7d",i,y);
  fprintf(fp2,"%d\n",y);
 }
 fclose(fp2);
 fp3=fopen(file3,"w");
 fprintf(fp3,"0.0\t %d\n",init);
 n=1;
 while (n<10001 && nonext[n]>0) {
  fprintf(fp3,"%5.3f\t %f\n",(double)n*d,(double)s[n]/(double)nonext[n]);
  ++n;
 }
 fclose(fp3);
 fp4=fopen(file4,"w");
 fprintf(fp4,"0.0\t %d\n",init);
 for(n=1;n<10001;++n)
 fprintf(fp4,"%5.3f\t %f\n",(double)n*d,(double)s[n]/(double)sample);
 fclose(fp4);
}
