   /* NormalizedCBP.c */
   /* Normalized continuous time branching process */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

double frand(void)
{
 return 1.0-(double)rand()/(RAND_MAX+1.0);
}

double mean(p)
 double p[5];
 {
  double mu=0.0;
  int i;
  for(i=1;i<5;++i) mu +=i*p[i];
  return mu;
 }
double var(p)
 double p[5];
 {
  double v=0.0;
  int i;
  for(i=1;i<5;++i) v+=i*i*p[i];
  return v-mean(p)*mean(p);
 }
int offspring(p,v)
 double p[5],v;
 {
  int i=0;
  double u=p[0];
  while ((i<4)&&(v>=u)) {
   ++i;u+=p[i];
  }
  return i;
 }
double c(t,a,m,s2)
 double t,a,m,s2;
 {
  double v;
  if (m==1.0) v=a*s2*t/2.0;
  else v=exp(a*(m-1.0)*t);
  return v;
 }

main(void)
{
 int i,
     init; /* initial size */
 long int seed,
          X; /* size at each time of the process */
 double a, /* parameter of exponential life time */
        m, /* mean offspring number */
        s2, /* variance of offspring */
        p[5], /* probability of giving birth to babies, p[1]=0 */
        time, /* time to be simulated */
        t; /* present time */
 char *file1 = "datax.dat",*file2 = "samplex.dat";
 FILE *fp1,*fp2;

   /*--------------- Input initial parameters ---------------*/

 printf(" Rate of an exponential life time (any positive number) = ");
 scanf("%lf",&a);
 printf("\n");
 printf(" Offspring distribution:\n\t");
  printf("p(0)=");
  scanf("%lf",&p[0]);
  printf("\t");
  p[1]=0.0;
 for(i=2;i<4;++i){
  printf("p(%d)=",i);
  scanf("%lf",&p[i]);
  printf("\t");
 }
 p[4]=1.0-p[0]-p[2]-p[3];
 printf("\n");m=mean(p);s2=var(p);
 printf(" Mean offspring number = %4.3f\n",m);
 printf("\n");
 printf(" Time to be observed (positive number) = ");
 scanf("%lf",&time);
 printf("\n");
 printf(" Initial population size (any positive integer) = ");
 scanf("%d",&init);
 printf("\n");
 printf(" Random number seed (any integer) = ");
 scanf("%d",&seed);
 printf("\n\n");

  /* ---- Print initial parameters in a file "datax.dat" ----*/

 fp1=fopen(file1,"w");
 fprintf(fp1,"     << Normalized continuous time branching process >>\n\n");
 fprintf(fp1," Rate of an exponential life time =%3.3f\n\n",a);
 fprintf(fp1," Offspring distribution:\n");
 for(i=0;i<5;++i) fprintf(fp1,"\tp(%d)=%3.3f",i,p[i]);
 fprintf(fp1,"\n\n");
 fprintf(fp1," Mean offspring number= %4.3f\n\n",m);
 fprintf(fp1," Variance of an offspring number= %4.3f\n\n",s2);
 fprintf(fp1," Time to be observed =%4.2f\n\n",time);
 fprintf(fp1," Initial population size = %d\n\n",init);
 fprintf(fp1," Random number seed = %d\n",seed);
 fclose(fp1);

  /* Start simulation to save the results in a file "samplex.dat" */

 fp2=fopen(file2,"w");
 srand(seed); /* giving the seed of rand */
 fprintf(fp2,"%5.3f\t %10d\n",0.0,init);
 t=0.0;X=init;
 while (t<time && X!=0) {
  t+=-log(frand())/(a*X);
  if (c(t,a,m,s2)==0.0) break;
  else {
   fprintf(fp2,"%f\t %f\n",t,X/(init*c(t,a,m,s2)));
   X+=offspring(p,frand())-1;
   fprintf(fp2,"%f\t %f\n",t,X/(init*c(t,a,m,s2)));
  }
 }
 fclose(fp2);
}
